/******************************************************************************
*
* (c) Copyright 2008 Jaroslav Necesany
* ALL RIGHTS RESERVED.
*
*******************************************************************************
*
* File      eeprom.h
*
* Author    Jaroslav Necesany
* 
* Version   01.0
* 
* Date      30-Sep-2008
* 
* Brief     This file contains the prototypes and defines for the EEPROM API.
*
******************************************************************************/

#ifndef _EEPROM_H
#define _EEPROM_H

/******************************************************************************
* Includes
******************************************************************************/

/******************************************************************************
* Constants
******************************************************************************/

/******************************************************************************
* Macros 
******************************************************************************/

/* Edit the following macro definition in order to select the EEPROM device used.
 * Available options are:
 *   EEPROM_24xx1025
 *   EEPROM_24xx512
 *   EEPROM_24xx256
 *   EEPROM_24xx128
 *   EEPROM_24xx64
 *   EEPROM_24xx32
 */
#define EEPROM_24xx512

/* The following macros defines the physical address of the device */
#define A0  0         /* Physical address bit 0 */
#define A1  0         /* Physical address bit 1 */
#define A2  1         /* Physical address bit 2 */
/* NOTE: When using 24xx1025, A2 MUST be "1". */
                                               
#if defined (EEPROM_24xx1025)
  #define EEPROM_PAGE_SIZE              128  /* EEPROM page size */
  #define EEPROM_PAGE_COUNT            1024  /* EEPROM page count */
  #define EEPROM_SIZE             (128*512)  /* EEPROM size of one block in Bytes */
  #define EEPROM_LAST_ADDRESS ((128*512)-1)  /* EEPROM highest address for one block */
  #define EEPROM_DIV_SHIFT                1

#elif defined (EEPROM_24xx512)
  #define EEPROM_PAGE_SIZE              128  /* EEPROM page size */
  #define EEPROM_PAGE_COUNT             512  /* EEPROM page count */
  #define EEPROM_SIZE             (128*512)  /* EEPROM size in Bytes */
  #define EEPROM_LAST_ADDRESS ((128*512)-1)  /* EEPROM highest address */
  #define EEPROM_DIV_SHIFT                1

#elif defined (EEPROM_24xx256)
  #define EEPROM_PAGE_SIZE               64  /* EEPROM page size */
  #define EEPROM_PAGE_COUNT             512  /* EEPROM page count */
  #define EEPROM_SIZE              (64*512)  /* EEPROM size in Bytes */
  #define EEPROM_LAST_ADDRESS  ((64*512)-1)  /* EEPROM highest address */
  #define EEPROM_DIV_SHIFT                2

#elif defined (EEPROM_24xx128)
  #define EEPROM_PAGE_SIZE               64  /* EEPROM page size */
  #define EEPROM_PAGE_COUNT             256  /* EEPROM page count */
  #define EEPROM_SIZE              (64*256)  /* EEPROM size in Bytes */
  #define EEPROM_LAST_ADDRESS  ((64*256)-1)  /* EEPROM highest address */
  #define EEPROM_DIV_SHIFT                2

#elif defined (EEPROM_24xx64)
  #define EEPROM_PAGE_SIZE               32  /* EEPROM page size */
  #define EEPROM_PAGE_COUNT             256  /* EEPROM page count */
  #define EEPROM_SIZE              (32*256)  /* EEPROM size in Bytes */
  #define EEPROM_LAST_ADDRESS  ((32*256)-1)  /* EEPROM highest address */
  #define EEPROM_DIV_SHIFT                3

#elif defined (EEPROM_24xx32)
  #define EEPROM_PAGE_SIZE               32  /* EEPROM page size */
  #define EEPROM_PAGE_COUNT             128  /* EEPROM page count */
  #define EEPROM_SIZE              (32*128)  /* EEPROM size in Bytes */
  #define EEPROM_LAST_ADDRESS  ((32*128)-1)  /* EEPROM highest address */
  #define EEPROM_DIV_SHIFT                3

#endif 

#define EEPROM_MOD_SHIFT   (16 - EEPROM_DIV_SHIFT)                
#define EEPROM_MUL_SHIFT   (EEPROM_MOD_SHIFT - (8 - EEPROM_DIV_SHIFT))


#if A0 == 1
  #define BIT_1 2
#else
  #define BIT_1 0
#endif

#if A1 == 1
  #define BIT_2 4
#else
  #define BIT_2 0
#endif

#if defined (EEPROM_24xx1025)
  #define BIT_3 0
#else 
  #if A2 == 1
    #define BIT_3 8
  #else
    #define BIT_3 0
  #endif
#endif

#define DEVICE_ADDRESS (160 + BIT_1 + BIT_2 + BIT_3)   

/******************************************************************************
* Types
******************************************************************************/

typedef enum 
{
  EEPROM_BLOCK_0, /* address range  0x0000 -  0xFFFF */
  EEPROM_BLOCK_1, /* address range 0x10000 - 0x1FFFF */
} EEPROM_BLOCK;

/******************************************************************************
* Global functions
******************************************************************************/

void EepromInit(unsigned char iicf);

void EepromReadPage(unsigned int pageNo, unsigned char buffer[EEPROM_PAGE_SIZE]);
void EepromWritePage(unsigned int pageNo, unsigned char buffer[EEPROM_PAGE_SIZE]);
    
void EepromReadCurrPageAndInc(unsigned char buffer[EEPROM_PAGE_SIZE]);  
void EepromWriteCurrPageAndInc(unsigned char buffer[EEPROM_PAGE_SIZE]); 
         
unsigned char EepromReadByteOfPage(unsigned int pageNo, unsigned char byteNo);
void EepromWriteByteOfPage(unsigned int pageNo, unsigned char byteNo, unsigned char byte);
         
unsigned char EepromReadByte(EEPROM_BLOCK block, unsigned int address); 
void EepromWriteByte(EEPROM_BLOCK block, unsigned int address, unsigned char byte);
         
unsigned char EepromReadCurrByteAndInc(void);       
void EepromWriteCurrByteAndInc(unsigned char byte); 
         
#endif /* _EEPROM_H */
